-- mthCounter
-- specialization script for FS 17 vehicles
-- adds analog operating time (in mth) counter
-- author: agRush2k
-- version 2.0.0.0
-- date: 2018-02-25 - 2018-03-09
-- changelog:
-- v2.0.0.0 - complete rebuild, adding new functions
-- v1.0.0.0 (2017-02-25) - public release

mthCounter = {};

function mthCounter.prerequisitesPresent (specializations)
	return
	SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function mthCounter:load(savegame)
	self.nominalRpm = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.mthCounter.nominalRpm#value"));

	self.hundredths = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.total.hundredths#index"));
	self.tenths = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.total.tenths#index"));
	self.ones = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.total.ones#index"));
	self.tens = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.total.tens#index"));
	self.hundreds = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.total.hundreds#index"));
	self.thousands = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.total.thousands#index"));
	-- operating time counter in one session
	self.hundredthsSession = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.oneSession.hundredths#index"));
	self.tenthsSession = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.oneSession.tenths#index"));
	self.onesSession = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.oneSession.ones#index"));
	self.tensSession = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.oneSession.tens#index"));
	self.hundredsSession = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.oneSession.hundreds#index"));
	self.thousandsSession = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.mthCounter.oneSession.thousands#index"));

	self.mth = 0;

	if savegame ~= nil then
		local operatingTime = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. "#operatingTime"), self.operatingTime) * 1000;
		local mth = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key .. "#mth"), self.mth) * 1000;
		-- WARNING: If the 'mth' attribute in savegame.xml is nil, the counter resets to zero
		self:setMth(mth, true);
		if mth ~= nil then
			self.mthrs = mth;
		else
			self.mthrs = operatingTime;
		end;
		local rotx = 2 * math.pi * self.mthrs / 3600000;
		if self.hundredths ~= nil then
			setRotation(self.hundredths, rotx * 10, 0, 0);
		end;
		if self.tenths ~= nil then
			setRotation(self.tenths, rotx, 0, 0);
		end;
		if self.ones ~= nil then
			setRotation(self.ones, rotx / 10, 0, 0);
		end;
		if self.tens ~= nil then
			setRotation(self.tens, rotx / 100, 0, 0);
		end;
		if self.hundreds ~= nil then
			setRotation(self.hundreds, rotx / 1000, 0, 0);
		end;
		if self.thousands ~= nil then
			setRotation(self.thousands, rotx / 10000, 0, 0);
		end;
	else
		self.mthrs = 0;
	end;
	local uiScale = g_gameSettings:getValue("uiScale")
    local mthBgWidth, mthBgHeight = getNormalizedScreenValues(83.5*uiScale, 27*uiScale)
    local mthBg2Width, mthBg2Height = getNormalizedScreenValues(76*uiScale, 21.5*uiScale)
    self.mthBg = Overlay:new("mthBg", g_baseUIFilename, 0.8045, 0, mthBgWidth, mthBgHeight);
    self.mthBg:setUVs(g_colorBgUVs);
    self.mthBg:setColor(unpack(g_colorBg));
    self.mthBg2 = Overlay:new("mthBg2", g_baseUIFilename, 0.8075, 0.005, mthBg2Width, mthBg2Height);
    self.mthBg2:setUVs(g_colorBgUVs);
    self.mthBg2:setColor(0.0075, 0.0075, 0.0075, 1);
	
	print("mthCounter: loaded ('"..self.configFileName.."')");
end;

function mthCounter:delete()
	self.mthBg:delete();
	self.mthBg2:delete();
end;

function mthCounter:readStream(streamId, connection)
    self:setMth(streamReadFloat32(streamId), true);
end;

function mthCounter:writeStream(streamId, connection)
    streamWriteFloat32(streamId, self.mth);
end;

function mthCounter:mouseEvent(posX, posY, isDown, isUp, button)
end;

function mthCounter:keyEvent(unicode, sym, modifier, isDown)
end;

function mthCounter:getSaveAttributesAndNodes(nodeIdent)
    local attributes = 'mth="'..tostring(self.mth / 1000)..'"';
    return attributes, nil;
end;

function mthCounter:update(dt)
	local rpmRotx = 2 * math.pi * self.mth / 3600000;
	local rpmRotx2 = 2 * math.pi * (self.mth - self.mthrs) / 3600000;
	if self.hundredths ~= nil and self.isMotorStarted then
		setRotation(self.hundredths, rpmRotx * 10, 0, 0);
	end;
	if self.hundredthsSession ~= nil and self.isMotorStarted then
		setRotation(self.hundredthsSession, rpmRotx2 * 10, 0, 0);
	end;
	if self.tenths ~= nil and self.isMotorStarted then
		setRotation(self.tenths, rpmRotx, 0, 0);
	end;
	if self.tenthsSession ~= nil and self.isMotorStarted then
		setRotation(self.tenthsSession, rpmRotx2, 0, 0);
	end;
	if self.ones ~= nil and self.isMotorStarted then
		setRotation(self.ones, rpmRotx / 10, 0, 0);
	end;
	if self.onesSession ~= nil and self.isMotorStarted then
		setRotation(self.onesSession, rpmRotx2 / 10, 0, 0);
	end;
	if self.tens ~= nil and self.isMotorStarted then
		setRotation(self.tens, rpmRotx / 100, 0, 0);
	end;
	if self.tensSession ~= nil and self.isMotorStarted then
		setRotation(self.tensSession, rpmRotx2 / 100, 0, 0);
	end;
	if self.hundreds ~= nil and self.isMotorStarted then
		setRotation(self.hundreds, rpmRotx / 1000, 0, 0);
	end;
	if self.hundredsSession ~= nil and self.isMotorStarted then
		setRotation(self.hundredsSession, rpmRotx2 / 1000, 0, 0);
	end;
	if self.thousands ~= nil and self.isMotorStarted then
		setRotation(self.thousands, rpmRotx / 10000, 0, 0);
	end;
	if self.thousandsSession ~= nil and self.isMotorStarted then
		setRotation(self.thousandsSession, rpmRotx2 / 10000, 0, 0);
	end;
end;

function mthCounter:updateTick(dt)
	local currentRpm = self.motor:getEqualizedMotorRpm();
	local timeScale = g_currentMission.missionInfo.timeScale;
	if self.nominalRpm ~= nil then
		self.rpm = self.nominalRpm;
	else
		self.rpm = self.motor:getMaxRpm();
	end;
	if self.isMotorStarted then
		self:setMth(self.mth + dt * (currentRpm / self.rpm) * timeScale);
	end;
end;

function Vehicle:setMth(mth, isLoading)
    self.mth = math.max(Utils.getNoNil(mth, 0), 0);
end;

function mthCounter:draw()
	if self.mth ~= nil then
		self.mthBg:render();
		self.mthBg2:render();
		local minutes = self.mth / (1000 * 60);
		local mths = math.floor(minutes / 60);
		minutes = math.floor((minutes - mths * 60) / 6);
		setTextAlignment(RenderText.ALIGN_RIGHT);
		renderText(0.84, 0.0126, getCorrectTextSize(0.02), mths .. ".");
        setTextColor(0.2832, 0.0091, 0.0091, 1)
		renderText(0.847, 0.0126, getCorrectTextSize(0.02), tostring(minutes));
        setTextColor(1, 1, 1, 1)
		renderText(0.865, 0.0126, getCorrectTextSize(0.015), "mth");
	end;
end;